@@ -29,6 +29,7 @@
 )]
 #![allow(clippy::multiple_crate_versions)]
 #![allow(clippy::redundant_pub_crate)]
+#![allow(clippy::option_if_let_else)]
 
 pub mod agg;
 pub mod binsearch;
@@ -40,7 +41,10 @@ pub mod matcher;
 pub mod num;
 mod shunting_yard;
 pub mod sort;
+pub mod tabs;
 pub mod text;
 mod tokenizer;
 pub mod tooltest;
+pub mod trans;
+pub mod transpose;
 pub mod util;
